@@ -27,6 +27,8 @@
 
 #include "clock.h"
 
+#include "egg-screen-help.h"
+
 #define INTERNETSECOND (864)
 #define INTERNETBEAT   (86400)
 
@@ -166,9 +168,9 @@ update_timeformat(ClockData *cd)
 
 	if (cd->hourformat == 12) {
 		if (cd->showseconds)
-			time = _("%I:%M:%S %p");
+			time = _("%l:%M:%S %p");
 		else
-			time = _("%I:%M %p");
+			time = _("%l:%M %p");
 	} else {
 		if (cd->showseconds)
 			time = _("%H:%M:%S");
@@ -176,21 +178,20 @@ update_timeformat(ClockData *cd)
 			time = _("%H:%M");
 	}
 
-	g_free (cd->timeformat);
 	if (cd->showdate) {
 		/* Show date in another line if panel is vertical, or
 		 * horizontal but large enough to hold two lines of text */
 		if (cd->orient == PANEL_APPLET_ORIENT_LEFT ||
 		    cd->orient == PANEL_APPLET_ORIENT_RIGHT ||
 		    cd->size >= GNOME_Vertigo_PANEL_MEDIUM)
-			loc = g_strconcat (time, "\n",
-					   _("%a %b %d"), NULL);
+			loc = g_strconcat (_("%a %b %d"), "\n", time, NULL);
 		else
-			loc = g_strconcat (time, " ",
-					   _("%a %b %d"), NULL);
+			loc = g_strconcat (_("%a %b %d"), ", ", time, NULL);
 	} else {
 		loc = g_strdup (time);
 	}
+
+	g_free (cd->timeformat);
 	cd->timeformat = g_locale_from_utf8 (loc, -1, NULL, NULL, NULL);
 	g_free (loc);
 }
@@ -366,6 +367,8 @@ destroy_clock(GtkWidget * widget, ClockData *cd)
 		cd->props = NULL;
 	}
 
+        g_free (cd->timeformat);
+        
 	g_free (cd);
 }
 
@@ -862,7 +865,9 @@ set_gmt_time_cb (GtkWidget *w,
 }
 
 static void
-properties_response_cb (GtkWidget *widget, gint id, gpointer data)
+properties_response_cb (GtkWidget *widget,
+			int        id,
+			ClockData *cd)
 {
 	
 	if (id == GTK_RESPONSE_HELP) {
@@ -877,8 +882,10 @@ properties_response_cb (GtkWidget *widget, gint id, gpointer data)
 							      GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
 		}
 
-		gnome_help_display_desktop (applet_program, "clock",
-					    "clock", "clock-settings", &error);
+		egg_screen_help_display_desktop (
+				gtk_widget_get_screen (cd->applet),
+				applet_program, "clock",
+				"clock", "clock-settings", &error);
 
 		if (error) {
 			GtkWidget *dialog;
@@ -894,6 +901,8 @@ properties_response_cb (GtkWidget *widget, gint id, gpointer data)
 					  NULL);
 
 			gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+			gtk_window_set_screen (GTK_WINDOW (dialog),
+					       gtk_widget_get_screen (cd->applet));
 			gtk_widget_show (dialog);
 			g_error_free (error);
 		}
@@ -925,7 +934,9 @@ display_properties_dialog (BonoboUIComponent *uic,
 	GSList    *list;
 	char      *file;
 
-	if (cd->props != NULL) {
+	if (cd->props) {
+		gtk_window_set_screen (GTK_WINDOW (cd->props),
+				       gtk_widget_get_screen (cd->applet));
 		gtk_window_present (GTK_WINDOW (cd->props));
 		return;
 	}
@@ -939,6 +950,8 @@ display_properties_dialog (BonoboUIComponent *uic,
 
 	gtk_dialog_set_has_separator (GTK_DIALOG (cd->props), FALSE);
 	gtk_dialog_set_default_response (GTK_DIALOG (cd->props), GTK_RESPONSE_CLOSE);
+	gtk_window_set_screen (GTK_WINDOW (cd->props),
+			       gtk_widget_get_screen (cd->applet));
 		
 	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
 	                                  "gnome-clock.png", TRUE, NULL);
@@ -978,7 +991,7 @@ display_properties_dialog (BonoboUIComponent *uic,
 	gtk_widget_show (twentyfourhour);
 	g_object_set_data (G_OBJECT (twentyfourhour), "user_data", cd);
 
-	unixtime = gtk_radio_button_new_with_mnemonic (gtk_radio_button_get_group (GTK_RADIO_BUTTON (twelvehour)), _("_UNIX time"));
+	unixtime = gtk_radio_button_new_with_mnemonic (gtk_radio_button_get_group (GTK_RADIO_BUTTON (twelvehour)), _("UNI_X time"));
 	gtk_box_pack_start (GTK_BOX (type_box), unixtime, FALSE, FALSE, 0);
 	gtk_widget_show (unixtime);
 
@@ -1078,7 +1091,7 @@ display_properties_dialog (BonoboUIComponent *uic,
 	g_signal_connect (G_OBJECT (cd->props), "destroy",
 			  G_CALLBACK (gtk_widget_destroyed), &(cd->props));
 	g_signal_connect (G_OBJECT (cd->props), "response",
-			  G_CALLBACK (properties_response_cb), NULL);
+			  G_CALLBACK (properties_response_cb), cd);
 
 	/* sets up atk relation  */
 	list = g_slist_append (NULL, twelvehour);
@@ -1110,8 +1123,9 @@ display_help_dialog (BonoboUIComponent *uic,
 						      GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
 	}
 
-	gnome_help_display_desktop (applet_program, "clock",
-				    "clock",NULL, &error);
+	egg_screen_help_display_desktop (
+			gtk_widget_get_screen (cd->applet),
+			applet_program, "clock", "clock",NULL, &error);
 	if (error) {
 		GtkWidget *dialog;
 		dialog = gtk_message_dialog_new (NULL,
@@ -1126,6 +1140,8 @@ display_help_dialog (BonoboUIComponent *uic,
 				  NULL);
 
 		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		gtk_window_set_screen (GTK_WINDOW (dialog),
+				       gtk_widget_get_screen (cd->applet));
 		gtk_widget_show (dialog);
 		g_error_free (error);
 	}
@@ -1154,8 +1170,9 @@ display_about_dialog (BonoboUIComponent *uic,
 	/* Translator credits */
 	const char *translator_credits = _("translator_credits");
 
-	if (about != NULL)
-	{
+	if (about) {
+		gtk_window_set_screen (GTK_WINDOW (about),
+				       gtk_widget_get_screen (cd->applet));
 		gtk_window_present (GTK_WINDOW (about));
 		return;
 	}
@@ -1179,6 +1196,8 @@ display_about_dialog (BonoboUIComponent *uic,
 				 pixbuf);
 	
 	gtk_window_set_wmclass (GTK_WINDOW (about), "clock", "Clock");
+	gtk_window_set_screen (GTK_WINDOW (about),
+			       gtk_widget_get_screen (cd->applet));
 
 	if (pixbuf) {
 		gtk_window_set_icon (GTK_WINDOW (about), pixbuf);
