@@ -161,13 +161,6 @@ struct _BasePPosClass {
 					    int *x, int *y,
 					    int w, int h);
 
-	void (*get_menu_pos)              (BasePWidget *basep,
-					   GtkWidget *widget,
-					   GtkRequisition *mreq,
-					   int *x, int *y,
-					   int wx, int wy,
-					   int ww, int wh);
-
 	void (*realize)                   (GtkWidget *widget);
 
 	void (*north_clicked)             (BasePWidget *basep);
@@ -217,6 +210,9 @@ void		basep_widget_change_params	(BasePWidget *basep,
 						 gboolean rotate_pixmap_bg,
 						 GdkColor *back_color);
 
+void            basep_widget_screen_size_changed (BasePWidget *basep,
+						  GdkScreen   *screen);
+
 /*gboolean       basep_widget_convert_to         (BasePWidget *basep,
 						 PanelType type);*/
 
@@ -254,12 +250,6 @@ void		basep_widget_get_position	(BasePWidget *basep,
 						 int *x, int *y,
 						 int w, int h);
 
-void            basep_widget_get_menu_pos  (BasePWidget *basep,
-					    GtkWidget *menu,
-					    int *x, int *y,
-					    int wx, int wy,
-					    int ww, int wh);
-
 PanelOrient basep_widget_get_applet_orient (BasePWidget *basep);
 
 /* initialize drag offsets according to cursor */
