@@ -38,6 +38,7 @@
 #include "egg-screen-exec.h"
 #include "panel-stock-icons.h"
 #include "panel-action-button.h"
+#include "panel-recent.h"
 
 #define ICON_SIZE 20
 #define FOOBAR_MENU_FLAGS (MAIN_MENU_SYSTEM | MAIN_MENU_KDE_SUB | MAIN_MENU_DISTRIBUTION_SUB)
@@ -181,6 +182,9 @@ append_actions_menu (FoobarWidget *foo,
 			  G_CALLBACK (panel_action_run_program), NULL);
 	setup_internal_applet_drag (item, "ACTION:run:NEW");
 
+	item = gtk_separator_menu_item_new ();
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
 	if (panel_is_program_in_path  ("gnome-search-tool")) {
 		item = stock_menu_item_new (
 				_("Search for Files..."),
@@ -198,6 +202,10 @@ append_actions_menu (FoobarWidget *foo,
 		setup_internal_applet_drag (item, "ACTION:search:NEW");
 	}
 
+	panel_recent_append_documents_menu (menu);
+	item = gtk_separator_menu_item_new ();
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
 	if (panel_is_program_in_path ("gnome-panel-screenshot")) {
 		item = stock_menu_item_new (_("Screenshot..."),
 					    PANEL_STOCK_SCREENSHOT,
@@ -801,6 +809,31 @@ foobar_widget_destroy (GtkObject *o)
 		GTK_OBJECT_CLASS (foobar_widget_parent_class)->destroy (o);
 }
 
+void
+foobar_widget_screen_size_changed (FoobarWidget *foo,
+				   GdkScreen    *screen)
+{
+	GtkWindow *window;
+	int        w, h;
+
+	window = GTK_WINDOW (foo);
+	
+	gtk_window_get_size (window, &w, &h);
+
+	gtk_window_move (window,
+			 multiscreen_x (foo->screen, foo->monitor),
+			 multiscreen_y (foo->screen, foo->monitor));
+
+	g_object_set (G_OBJECT (foo), "width_request",
+		      multiscreen_width (foo->screen, foo->monitor),
+		      NULL);
+
+	gtk_window_set_resizable (window, TRUE);
+	gtk_window_resize (window,
+			   multiscreen_width (foo->screen, foo->monitor),
+			   h);
+}
+
 static void
 foobar_widget_size_allocate (GtkWidget *w, GtkAllocation *alloc)
 {
