@@ -46,6 +46,8 @@
 #include "panel-gconf.h"
 #include "quick-desktop-reader.h"
 #include "nothing.h"
+#include "egg-screen-exec.h"
+#include "egg-screen-url.h"
 #include "panel-stock-icons.h"
 
 enum {
@@ -154,23 +156,35 @@ kill_completion (void)
 }
 
 static void
-get_environment (int *argc, char ***argv, int *envc, char ***envv)
+get_environment (int         *argc,
+		 char      ***argv,
+		 int         *envc,
+		 char      ***envv,
+		 GdkScreen   *screen)
 {
-	GList *envar = NULL, *li;
-	int i, moveby;
+	GList    *envar = NULL, *li;
+	gboolean  display_found = FALSE;
+	int       i, moveby;
 
 	*envv = NULL;
 	*envc = 0;
 
 	moveby = 0;
 	for (i = 0; i < *argc; i++) {
-		if (strchr ((*argv)[i], '=') == NULL) {
+		if (!strchr ((*argv) [i], '='))
 			break;
-		}
+
+		if (!strncmp ((*argv) [i], "DISPLAY", 7))
+			display_found = TRUE;
+
 		envar = g_list_append (envar, g_strdup ((*argv)[i]));
 		moveby ++;
 	}
 
+	if (!display_found && gdk_screen_get_default () != screen)
+		envar = g_list_append (
+				envar, egg_screen_exec_display_string (screen));
+
 	if (moveby == *argc) {
 		panel_g_list_deep_free (envar);
 		return;
@@ -179,7 +193,7 @@ get_environment (int *argc, char ***argv, int *envc, char ***envv)
 	if (envar == NULL)
 		return;
 
-	for (i = 0; i < *argc; i++) {
+	for (i = 0; i < *argc && moveby; i++) {
 		g_free ((*argv)[i]);
 		if (i + moveby < *argc) {
 			(*argv)[i] = (*argv)[i+moveby];
@@ -224,10 +238,11 @@ launch_selected (GtkTreeModel *model,
 	g_free (name);
 
 	if (!ditem) {
-		panel_error_dialog ("failed_to_load_desktop",
-				    _("<b>Failed to run this program</b>\n\n"
-				      "Details: %s"),
-				    error->message);
+		panel_error_dialog (
+			gtk_window_get_screen (GTK_WINDOW (run_dialog)),
+			"failed_to_load_desktop",
+			_("<b>Failed to run this program</b>\n\nDetails: %s"),
+			error->message);
 		g_clear_error (&error);
 		return;
 	}
@@ -240,11 +255,14 @@ launch_selected (GtkTreeModel *model,
 					GNOME_DESKTOP_ITEM_TERMINAL,
 					terminal->active);
 
-	if (!gnome_desktop_item_launch (ditem, NULL, 0, &error)) {
-		panel_error_dialog ("failed_to_load_desktop",
-				    _("<b>Failed to run this program</b>\n\n"
-				      "Details: %s"),
-				    error->message);
+	if (!panel_ditem_launch (
+			gtk_window_get_screen (GTK_WINDOW (run_dialog)),
+			ditem, NULL, 0, &error)) {
+		panel_error_dialog (
+			gtk_window_get_screen (GTK_WINDOW (run_dialog)),
+			"failed_to_load_desktop",
+			_("<b>Failed to run this program</b>\n\nDetails: %s"),
+			error->message);
 		g_clear_error (&error);
 	}
 
@@ -269,12 +287,15 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 	int argc, temp_argc;
 	char *s = NULL;
 	char *escaped = NULL;
+	char *disk = NULL;
 	char **envv = NULL;
 	int envc;
 	GError *error = NULL;
 
 	if (response == GTK_RESPONSE_HELP) {
-		panel_show_help ("wgoseditmainmenu.xml", "gospanel-23");
+		panel_show_help (
+			gtk_window_get_screen (GTK_WINDOW (w)),
+			"wgoseditmainmenu.xml", "gospanel-23");
 		/* just return as we don't want to close */
 		return;
 	} else if (response != PANEL_RESPONSE_RUN) {
@@ -302,28 +323,34 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
                 entry = g_object_get_data (G_OBJECT (w), "entry");
 
                 s = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
-		escaped = g_markup_escape_text (s, -1);
 
                 if (string_empty (s))
                         goto return_and_close;
 
+		escaped = g_markup_escape_text (s, -1);
+		disk = g_filename_from_utf8 (s, -1, NULL, NULL, NULL);
+
 		/* save command in history */
 		gnome_entry = g_object_get_data (G_OBJECT (w), "gnome_entry");
 		gnome_entry_prepend_history (gnome_entry, TRUE, s);
 
                 /* evil eggies, do not translate! */
                 if (strcmp (s, "you shall bring us a shrubbery") == 0) {
-                        panel_info_dialog ("ni_ni_ni_ni",
-					   "NI! NI! NI! NI! NI! NI!");
+                        panel_info_dialog (
+				gtk_window_get_screen (GTK_WINDOW (run_dialog)),
+				"ni_ni_ni_ni",
+				"NI! NI! NI! NI! NI! NI!");
                         goto return_and_close;
                 } else if (strcmp (s, "supreme executive power") == 0) {
-                        panel_info_dialog ("evil",
-					   "Listen -- strange women lying in\n"
-					   "ponds distributing swords is no\n"
-					   "basis for a system of government.\n"
-					   "Supreme executive power derives from\n"
-					   "a mandate from the masses, not from\n"
-					   "some farcical aquatic ceremony!");
+                        panel_info_dialog (
+				gtk_window_get_screen (GTK_WINDOW (run_dialog)),
+				"evil",
+				"Listen -- strange women lying in\n"
+				"ponds distributing swords is no\n"
+				"basis for a system of government.\n"
+				"Supreme executive power derives from\n"
+				"a mandate from the masses, not from\n"
+				"some farcical aquatic ceremony!");
                         goto return_and_close;
                 } else if (strcmp (s, "free the fish") == 0) {
 			start_screen_check ();
@@ -332,25 +359,36 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 			start_geginv ();
                         goto return_and_close;
 		} else if (strcmp (s, "End world hunger") == 0) {
-			gnome_url_show ("http://www.wfp.org", NULL);
+			egg_screen_url_show (
+				gtk_window_get_screen (GTK_WINDOW (run_dialog)),
+				"http://www.wfp.org", NULL);
                         goto return_and_close;
 		}
 
                 /* Somewhat of a hack I suppose */
                 if (panel_is_url (s)) {
-                        gnome_url_show (s, NULL);
+			/* FIXME: URLs are in UTF8 ... right? */
+                        egg_screen_url_show (
+				gtk_window_get_screen (GTK_WINDOW (run_dialog)), s, NULL);
                         goto return_and_close;
                 }
 
-                if ( ! g_shell_parse_argv (s, &temp_argc, &temp_argv, &error)) {
-			panel_error_dialog ("run_error",
-					    _("<b>Failed to execute command:</b> '%s'\n\nDetails: %s"),
-					    escaped, error->message);
+		/* Note, the command is taken to have to be in disk encoding
+		 * even though it could contain strings, but more likely
+		 * it is all filenames and thus should be in disk encoding */
+                if ( ! g_shell_parse_argv (disk, &temp_argc, &temp_argv, &error)) {
+			panel_error_dialog (
+				gtk_window_get_screen (GTK_WINDOW (run_dialog)),
+				"run_error",
+				_("<b>Failed to execute command:</b> '%s'\n\nDetails: %s"),
+				escaped, error->message);
 			g_clear_error (&error);
                         goto return_and_close;
                 }
 
-                get_environment (&temp_argc, &temp_argv, &envc, &envv);
+                get_environment (
+			&temp_argc, &temp_argv, &envc, &envv,
+			gtk_window_get_screen (GTK_WINDOW (run_dialog)));
 
 		terminal = GTK_TOGGLE_BUTTON (
 				g_object_get_data (G_OBJECT (w), "terminal"));
@@ -417,9 +455,11 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 			
 			info = gnome_vfs_file_info_new ();
 			if (gnome_vfs_get_file_info (path, info, 0) != GNOME_VFS_OK) {
-				panel_error_dialog("run_error",
-						   _("<b>Failed to execute command:</b> '%s'\n\nDetails: %s"),
-                	                           escaped, g_strerror (errno));
+				panel_error_dialog(
+					gtk_window_get_screen (GTK_WINDOW (run_dialog)),
+					"run_error",
+					_("<b>Failed to execute command:</b> '%s'\n\nDetails: %s"),
+					escaped, g_strerror (errno));
 				g_free (path);
 				gnome_vfs_file_info_unref (info);
 				goto return_and_close;
@@ -440,9 +480,12 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 				g_free (mime_info);				
 				
 				if (command == NULL) {
-					panel_error_dialog ("run_error",
-							    _("<b>Failed to open file:</b> '%s'\n\nDetails: no application available to open file"),
-							    escaped);
+					panel_error_dialog (
+						gtk_window_get_screen (GTK_WINDOW (run_dialog)),
+						"run_error",
+						_("<b>Failed to open file:</b> '%s'\n\n"
+						  "Details: no application available to open file"),
+						escaped);
 
 					gnome_vfs_file_info_unref (info);
 					g_free (path);
@@ -453,10 +496,13 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 				command = g_strconcat ("nautilus ", path, NULL);
 			}
 			
-			if (!g_spawn_command_line_async (command, &error)) {
-				panel_error_dialog ("run_error",
-						    _("<b>Failed to open file:</b> '%s'\n\nDetails: %s"),
-						    escaped, error->message);
+			if (!egg_screen_execute_command_line_async (
+					gtk_window_get_screen (GTK_WINDOW (run_dialog)), command, &error)) {
+				panel_error_dialog (
+					gtk_window_get_screen (GTK_WINDOW (run_dialog)),
+					"run_error",
+					_("<b>Failed to open file:</b> '%s'\n\nDetails: %s"),
+					escaped, error->message);
 				g_clear_error (&error);
 			}
 	
@@ -472,6 +518,7 @@ return_and_close:
 	g_strfreev (envv);
 	g_free (s);
 	g_free (escaped);
+	g_free (disk);
 
 	gtk_widget_destroy (w);
         
@@ -494,7 +541,7 @@ quote_string (const char *s)
 }
 
 static void
-append_file (GtkWidget *entry, const char *file)
+append_file_utf8 (GtkWidget *entry, const char *file)
 {
 	const char *text;
 	char *quoted = quote_string (file);
@@ -509,6 +556,16 @@ append_file (GtkWidget *entry, const char *file)
 	g_free (quoted);
 }
 
+static void
+append_file (GtkWidget *entry, const char *file)
+{
+	char *utf8_file = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+	if (utf8_file != NULL) {
+		append_file_utf8 (entry, utf8_file);
+		g_free (utf8_file);
+	}
+}
+
 static void
 browse_ok (GtkWidget *widget, GtkFileSelection *fsel)
 {
@@ -553,8 +610,6 @@ browse (GtkWidget *w, GtkWidget *entry)
 		 G_CALLBACK (gtk_widget_destroy),
 		 G_OBJECT (fsel));
 
-	gtk_window_set_position (GTK_WINDOW (fsel), GTK_WIN_POS_MOUSE);
-
 	gtk_window_present (GTK_WINDOW (fsel));
 }
 
@@ -846,11 +901,13 @@ drag_data_received (GtkWidget        *widget,
 	for (i = 0; uris[i] != NULL; i++) {
 		char *file = gnome_vfs_get_local_path_from_uri (uris[i]);
 
+		/* FIXME: I assume the file is in utf8 encoding if coming
+		 * from a URI? */
 		if (file != NULL) {
-			append_file (entry, file);
+			append_file_utf8 (entry, file);
 			g_free (file);
 		} else {
-			append_file (entry, uris[i]);
+			append_file_utf8 (entry, uris[i]);
 		}
 	}
 
@@ -862,7 +919,7 @@ drag_data_received (GtkWidget        *widget,
 #define ELEMENTS(x) (sizeof (x) / sizeof (x[0]))
 
 static GtkWidget*
-create_simple_contents (void)
+create_simple_contents (GdkScreen *screen)
 {
         GtkWidget *vbox;
         GtkWidget *entry;
@@ -874,6 +931,7 @@ create_simple_contents (void)
         GtkWidget *w;
 	const char *key;
 	gboolean enable_program_list;
+	int width_request;
 	static GtkTargetEntry drop_types[] = { { "text/uri-list", 0, 0 } };
         
         vbox = gtk_vbox_new (FALSE, 0);
@@ -898,9 +956,9 @@ create_simple_contents (void)
         gtk_box_pack_start (GTK_BOX (vbox2), gentry,
 			    TRUE, TRUE, GNOME_PAD_SMALL);
 
-        /* 1/4 the width of the first screen should be a good value */
-	g_object_set (G_OBJECT (gentry),"width_request",
-		      (int)(multiscreen_width (0) / 4), NULL);
+        /* 1/4 the width of the first monitor should be a good value */
+	width_request = multiscreen_width (gdk_screen_get_number (screen), 0) / 4;
+	g_object_set (G_OBJECT (gentry), "width_request", width_request, NULL);
 
         entry = gnome_entry_gtk_entry (GNOME_ENTRY (gentry));
 	gtk_tooltips_set_tip (panel_tooltips, entry, _("Command to run"), NULL);
@@ -1113,7 +1171,7 @@ find_icon_timeout (gpointer data)
 
 	pixbuf = NULL;
 	if (found_icon != NULL) {
-		icon = gnome_desktop_item_find_icon (panel_icon_loader,
+		icon = gnome_desktop_item_find_icon (panel_icon_theme,
                                                      found_icon,
 						     48 /* desired size */,
 						     0 /* flags */);
@@ -1402,7 +1460,7 @@ selection_changed (GtkTreeSelection *selection,
 				gtk_label_set_text (GTK_LABEL (desc_label),
 						    sure_string (qitem->comment));
 
-			icon = gnome_desktop_item_find_icon (panel_icon_loader,
+			icon = gnome_desktop_item_find_icon (panel_icon_theme,
                                                              qitem->icon,
 							     48 /* desired size */,
 							     0 /* flags */);
@@ -1562,7 +1620,7 @@ run_dialog_destroyed (GtkWidget *widget)
 }
 
 void
-show_run_dialog (void)
+show_run_dialog (GdkScreen *screen)
 {
         gboolean  enable_program_list;
 	GtkWidget *w;
@@ -1572,7 +1630,8 @@ show_run_dialog (void)
 	if (no_run_box)
 		return;
 
-	if (run_dialog != NULL) {
+	if (run_dialog) {
+		gtk_window_set_screen (GTK_WINDOW (run_dialog), screen);
 		gtk_window_present (GTK_WINDOW (run_dialog));
 		/* always focus the entry initially */
 		w = g_object_get_data (G_OBJECT (run_dialog), "entry");
@@ -1580,16 +1639,15 @@ show_run_dialog (void)
 		return;
 	}
 
-	run_dialog = gtk_dialog_new_with_buttons (_("Run Program"),
-						  NULL /* parent */,
-						  0 /* flags */,
-						  GTK_STOCK_HELP,
-						  GTK_RESPONSE_HELP,
-						  GTK_STOCK_CANCEL,
-						  GTK_RESPONSE_CANCEL,
-						  NULL);
+	run_dialog = gtk_dialog_new_with_buttons (
+				_("Run Program"),
+				NULL, 0 /* flags */,
+				GTK_STOCK_HELP, GTK_RESPONSE_HELP,
+				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				NULL);
 
 	gtk_window_set_resizable (GTK_WINDOW (run_dialog), FALSE);
+	gtk_window_set_screen (GTK_WINDOW (run_dialog), screen);
 
 	w = gtk_dialog_add_button (GTK_DIALOG (run_dialog),
 				   PANEL_STOCK_EXECUTE, PANEL_RESPONSE_RUN);
@@ -1607,7 +1665,6 @@ show_run_dialog (void)
 			  G_CALLBACK (run_dialog_destroyed),
 			  NULL);
 
-	gtk_window_set_position (GTK_WINDOW (run_dialog), GTK_WIN_POS_MOUSE);
 	gtk_window_set_wmclass (GTK_WINDOW (run_dialog), "run_dialog", "Panel");
 
 	gtk_dialog_set_default_response (GTK_DIALOG (run_dialog), PANEL_RESPONSE_RUN);
@@ -1616,7 +1673,7 @@ show_run_dialog (void)
 			  G_CALLBACK (run_dialog_response), NULL);
 
 
-        create_simple_contents ();
+        create_simple_contents (screen);
 	
 	key = panel_gconf_general_key
 		(panel_gconf_get_profile (), "enable_program_list"),
@@ -1641,13 +1698,15 @@ show_run_dialog (void)
 }
 
 void
-show_run_dialog_with_text (const char *text)
+show_run_dialog_with_text (GdkScreen  *screen,
+			   const char *text)
 {
 	GtkWidget *entry;
+	char *exec;
 
 	g_return_if_fail (text != NULL);
 
-	show_run_dialog ();
+	show_run_dialog (screen);
 
 	if (run_dialog == NULL) {
 		return;
@@ -1655,5 +1714,9 @@ show_run_dialog_with_text (const char *text)
         
 	entry = g_object_get_data (G_OBJECT (run_dialog), "entry");
 
-	gtk_entry_set_text (GTK_ENTRY (entry), text);
+	exec = remove_parameters (text);
+
+	gtk_entry_set_text (GTK_ENTRY (entry), exec);
+
+	g_free (exec);
 }
