@@ -112,6 +112,24 @@ panel_gconf_full_key (PanelGConfKeyType  type,
 			profile, subdir, id, key);
 }
 
+gboolean
+panel_gconf_string_to_enum (GConfEnumStringPair  lookup_table [],
+			    const char          *str,
+			    int                 *enum_value_retloc,
+			    int                  fallback_value)
+{
+	g_return_val_if_fail (lookup_table != NULL, FALSE);
+	g_return_val_if_fail (enum_value_retloc != NULL, FALSE);
+
+	if (!str) {
+		*enum_value_retloc = fallback_value;
+		return FALSE;
+	}
+
+	return gconf_string_to_enum (lookup_table, str, enum_value_retloc);
+}
+
+
 GConfClient * 
 panel_gconf_get_client (void)
 {
@@ -404,11 +422,11 @@ panel_gconf_setup_profile (const char *profile)
 	schema_dir = g_strconcat ("/schemas/apps/panel/default_profiles/", "medium", NULL);
 
 	panel_gconf_associate_schemas_in_dir (
-		panel_gconf_client, profile_dir, schema_dir, &error);
+		panel_gconf_get_client (), profile_dir, schema_dir, &error);
 
-	if (error) {
+	if (error != NULL) {
 		g_warning ("gconf error: %s", error->message);
-		g_error_free (error);
+		g_clear_error (&error);
 	}
 
 	g_free (profile_dir);
