@@ -4,9 +4,11 @@
  * Copyright 2000 Helix Code, Inc.
  * Copyright 2000,2001 Eazel, Inc.
  * Copyright 2001 George Lebl
+ * Copyright 2002 Sun Microsystems Inc.
  *
  * Authors: George Lebl
  *          Jacob Berkman
+ *          Mark McLoughlin
  */
 
 #include <config.h>
@@ -34,26 +36,68 @@
 
 #include "applet.h"
 #include "nothing.h"
+#include "basep-widget.h"
+#include "foobar-widget.h"
+#include "panel.h"
+#include "egg-screen-exec.h"
+#include "egg-screen-url.h"
+#include "egg-screen-help.h"
 
 extern GlobalConfig global_config;
 
 extern GSList *applets;
+extern GSList *panels;
+
+GdkScreen *
+panel_screen_from_number (int screen)
+{
+	return gdk_display_get_screen (
+			gdk_display_get_default (), screen);
+}
+
+int
+panel_ditem_launch (GdkScreen                    *screen,
+		    const GnomeDesktopItem       *item,
+		    GList                        *file_list,
+		    GnomeDesktopItemLaunchFlags   flags,
+		    GError                      **error)
+{
+	char **envp = NULL;
+	int    retval;
+
+	if (gdk_screen_get_default () != screen)
+		envp = egg_screen_exec_environment (screen);
+
+	retval = gnome_desktop_item_launch_with_env (
+			item, file_list, flags, envp, error);
+
+	g_strfreev (envp);
+
+	return retval;
+}
 
 void
-panel_show_help (const char *doc_name, const char *linkid)
+panel_show_help (GdkScreen  *screen,
+		 const char *doc_name,
+		 const char *linkid)
 {
 	GError *error = NULL;
 
-	if (!gnome_help_display_desktop (NULL, "user-guide", doc_name, linkid, &error)) {
-		panel_error_dialog ("cannot_show_help",
-				    _("<b>Cannot display help document</b>\n\n"
-				      "Details: %s"), error != NULL ? error->message : "");
+	if (!egg_screen_help_display_desktop (screen, NULL, "user-guide", doc_name, linkid, &error)) {
+		panel_error_dialog (
+			screen,
+			"cannot_show_help",
+			_("<b>Cannot display help document</b>\n\n"
+			  "Details: %s"),
+			error != NULL ? error->message : "");
 		g_clear_error (&error);
 	}
 }
 
 static gboolean
-panel_show_gnome_help (const char *docpath, GError **error)
+panel_show_gnome_help (GdkScreen   *screen,
+		       const char  *docpath,
+		       GError     **error)
 {
 	char *app, *p, *path;
 	gboolean retval;
@@ -78,16 +122,9 @@ panel_show_gnome_help (const char *docpath, GError **error)
 
 	retval = TRUE;
 
-	if ( ! gnome_help_display_desktop (NULL /* program */,
-					   app /* doc_id */,
-					   path /* file_name */,
-					   NULL /* link_id */,
-					   error)) {
-		retval = gnome_help_display_with_doc_id (NULL /* program */,
-							 app /* doc_id */,
-							 path /* file_name */,
-							 NULL /* link_id */,
-							 error);
+	if ( ! egg_screen_help_display_desktop (screen, NULL, app, path, NULL, error)) {
+		retval = egg_screen_help_display_with_doc_id (
+				screen, NULL, app, path, NULL, error);
 	}
 
 	g_free (app);
@@ -96,7 +133,9 @@ panel_show_gnome_help (const char *docpath, GError **error)
 }
 
 static gboolean
-panel_show_kde_help (const char *docpath, GError **error)
+panel_show_kde_help (GdkScreen   *screen,
+		     const char  *docpath,
+		     GError     **error)
 {
 	const GList *li;
 
@@ -119,7 +158,7 @@ panel_show_kde_help (const char *docpath, GError **error)
 			gboolean retval;
 			char *uri = g_strconcat ("ghelp:", fullpath, NULL);
 			g_free (fullpath);
-			retval = gnome_help_display_uri (uri, error);
+			retval = egg_screen_help_display_uri (screen, uri, error);
 			g_free (uri);
 			return retval;
 		}
@@ -135,8 +174,9 @@ panel_show_kde_help (const char *docpath, GError **error)
 }
 
 gboolean
-panel_show_gnome_kde_help (const char *docpath,
-			   GError **error)
+panel_show_gnome_kde_help (GdkScreen   *screen,
+			   const char  *docpath,
+			   GError     **error)
 {
 	if (string_empty (docpath)) {
 		g_set_error (error,
@@ -147,10 +187,10 @@ panel_show_gnome_kde_help (const char *docpath,
 	}
 
 	if (panel_is_url (docpath))
-		return gnome_help_display_uri (docpath, error);
+		return egg_screen_help_display_uri (screen, docpath, error);
 
-	if ( ! panel_show_gnome_help (docpath, error)) {
-		return panel_show_kde_help (docpath, error);
+	if ( ! panel_show_gnome_help (screen, docpath, error)) {
+		return panel_show_kde_help (screen, docpath, error);
 	}
 
 	return TRUE;
@@ -601,36 +641,38 @@ convert_keysym_state_to_string(guint keysym,
 }
 
 static GtkWidget *
-panel_dialog (GtkWidget *parent,
-	      int type,
+panel_dialog (GdkScreen  *screen,
+	      int         type,
 	      const char *class,
 	      const char *str)
 {
-	GtkWidget *w;
+	GtkWidget *dialog;
 
-	w = gtk_message_dialog_new ((GtkWindow *) parent, 0, type,
-				    GTK_BUTTONS_OK, "foo");
-	gtk_widget_add_events (w, GDK_KEY_PRESS_MASK);
-	g_signal_connect (G_OBJECT (w), "event",
+	dialog = gtk_message_dialog_new (
+			NULL, 0, type, GTK_BUTTONS_OK, "foo");
+	gtk_widget_add_events (dialog, GDK_KEY_PRESS_MASK);
+	g_signal_connect (dialog, "event",
 			  G_CALLBACK (panel_dialog_window_event), NULL);
-	gtk_window_set_wmclass (GTK_WINDOW (w),
-				class, "Panel");
+	gtk_label_set_markup (
+		GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label), str);
 
-	gtk_label_set_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (w)->label), str);
+	gtk_window_set_wmclass (GTK_WINDOW (dialog), class, "Panel");
+	gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 
-	gtk_widget_show_all (w);
+	gtk_widget_show_all (dialog);
 
 	/* FIXME: this is ugly and makes it bad to run gtk_dialog_run
 	 * after this function */
-	g_signal_connect_swapped (G_OBJECT (w), "response",
+	g_signal_connect_swapped (G_OBJECT (dialog), "response",
 				  G_CALLBACK (gtk_widget_destroy),
-				  G_OBJECT (w));
+				  G_OBJECT (dialog));
 
-	return w;
+	return dialog;
 }
 
 GtkWidget *
-panel_error_dialog (const char *class,
+panel_error_dialog (GdkScreen  *screen,
+		    const char *class,
 		    const char *format,
 		    ...)
 {
@@ -647,13 +689,14 @@ panel_error_dialog (const char *class,
 		va_end (ap);
 	}
 
-	w = panel_dialog (NULL, GTK_MESSAGE_ERROR, class, s);
+	w = panel_dialog (screen, GTK_MESSAGE_ERROR, class, s);
 	g_free (s);
 	return w;
 }
 
 GtkWidget *
-panel_info_dialog (const char *class,
+panel_info_dialog (GdkScreen  *screen,
+		   const char *class,
 		   const char *format,
 		   ...)
 {
@@ -670,55 +713,7 @@ panel_info_dialog (const char *class,
 		va_end (ap);
 	}
 
-	w = panel_dialog (NULL, GTK_MESSAGE_INFO, class, s);
-	g_free (s);
-	return w;
-}
-
-GtkWidget *
-panel_error_dialog_with_parent (GtkWindow *parent,
-				const char *class,
-				const char *format,
-				...)
-{
-	GtkWidget *w;
-	char *s;
-	va_list ap;
-
-	if (format == NULL) {
-		g_warning ("NULL error dialog");
-		s = g_strdup ("(null)");
-	} else {
-		va_start (ap, format);
-		s = g_strdup_vprintf (format, ap);
-		va_end (ap);
-	}
-
-	w = panel_dialog (GTK_WIDGET (parent), GTK_MESSAGE_ERROR, class, s);
-	g_free (s);
-	return w;
-}
-
-GtkWidget *
-panel_info_dialog_with_parent (GtkWindow *parent,
-			       const char *class,
-			       const char *format,
-			       ...)
-{
-	GtkWidget *w;
-	char *s;
-	va_list ap;
-
-	if (format == NULL) {
-		g_warning ("NULL info dialog");
-		s = g_strdup ("(null)");
-	} else {
-		va_start (ap, format);
-		s = g_strdup_vprintf (format, ap);
-		va_end (ap);
-	}
-
-	w = panel_dialog (GTK_WIDGET (parent), GTK_MESSAGE_INFO, class, s);
+	w = panel_dialog (screen, GTK_MESSAGE_INFO, class, s);
 	g_free (s);
 	return w;
 }
@@ -1458,3 +1453,18 @@ missing_pixbuf (int size)
 
 	return pb;
 }
+
+void
+panel_lock_screen (GdkScreen *screen)
+{
+	char *argv[3] = {"xscreensaver-command", "-lock", NULL};
+
+	if (!screen)
+		screen = gdk_screen_get_default ();
+
+	if (egg_screen_execute_async (screen, g_get_home_dir (), 2, argv) < 0)
+		panel_error_dialog (screen,
+				    "cannot_exec_xscreensaver",
+				    _("<b>Cannot execute xscreensaver</b>\n\n"
+				    "Details: xscreensaver-command not found"));
+}
